"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$json = exports.$fusion = exports.$includesdict = exports.$dict = exports.$partial = exports.$keys = exports.$jsonobj = exports.$map = exports.$arraybuffer = exports.$lengthin = exports.$length = exports.$count = exports.$ascii = exports.$fpad = exports.$fpad4 = exports.$fpad3 = exports.$fpad2 = exports.$trim = exports.$totype = exports.$strings = exports.$string = exports.$div = exports.$unsigned = exports.$unsignedornull = exports.$address = exports.$isodate = exports.$UUID = exports.$url = exports.$email = exports.$int = exports.$intornull = exports.$isfunction = exports.$isuuid = exports.$isurl = exports.$isemail = exports.$isdate = exports.$isarray = exports.$objectcount = exports.$isobject = exports.$isbool = exports.$isunsigned = exports.$isint = exports.$isnumber = exports.$isstring = exports.$ok = exports.$defined = void 0;
const compare_1 = require("./compare");
const string_tables_1 = require("./string_tables");
const tsdate_1 = require("./tsdate");
const tsdatecomp_1 = require("./tsdatecomp");
const tsdefaults_1 = require("./tsdefaults");
const types_1 = require("./types");
function $defined(o) { return o !== undefined && typeof o !== 'undefined'; }
exports.$defined = $defined;
function $ok(o) { return o !== null && o !== undefined && typeof o !== 'undefined'; }
exports.$ok = $ok;
function $isstring(o) { return o !== null && o !== undefined && typeof o === 'string'; }
exports.$isstring = $isstring;
function $isnumber(o) { return o !== null && o !== undefined && typeof o === 'number' && !isNaN(o) && isFinite(o); }
exports.$isnumber = $isnumber;
function $isint(o) { return o !== null && o !== undefined && typeof o === 'number' && Number.isSafeInteger(o) && o >= types_1.INT_MIN && o <= types_1.INT_MAX; }
exports.$isint = $isint;
function $isunsigned(o) { return o !== null && o !== undefined && typeof o === 'number' && Number.isSafeInteger(o) && o >= 0 && o <= types_1.UINT_MAX; }
exports.$isunsigned = $isunsigned;
function $isbool(o) { return o !== null && o !== undefined && typeof o === 'boolean'; }
exports.$isbool = $isbool;
function $isobject(o) { return o !== null && o !== undefined && typeof o === 'object'; }
exports.$isobject = $isobject;
function $objectcount(o) { return $isobject(o) ? $keys(o).length : 0; }
exports.$objectcount = $objectcount;
function $isarray(o) { return o !== null && o !== undefined && Array.isArray(o); }
exports.$isarray = $isarray;
function $isdate(o) { return o instanceof Date || o instanceof tsdate_1.TSDate || ($isstring(o) && $ok($isodate(o))); }
exports.$isdate = $isdate;
function $isemail(o) { return $isstring(o) && $ok($email(o)); }
exports.$isemail = $isemail;
function $isurl(o, opts) { return o instanceof URL || ($isstring(o) && $ok($url(o, opts))); }
exports.$isurl = $isurl;
function $isuuid(o) { return $isstring(o) && $ok($UUID(o)); }
exports.$isuuid = $isuuid;
function $isfunction(o) { return typeof o === 'function'; }
exports.$isfunction = $isfunction;
function $intornull(n) {
    if (!$ok(n)) {
        return null;
    }
    if (typeof n === 'string') {
        n = parseInt(n, 10);
    }
    return $isint(n) ? n : null;
}
exports.$intornull = $intornull;
function $int(n, defaultValue = 0) {
    n = $intornull(n);
    return $ok(n) ? n : defaultValue;
}
exports.$int = $int;
function $email(s) {
    const m = _regexvalidatedstring(types_1.emailRegex, s);
    return $ok(m) ? m.toLowerCase() : null;
}
exports.$email = $email;
function $url(s, opts = {}) {
    if (!$length(s)) {
        return null;
    }
    const m = s.match(types_1.urlRegex);
    if ((m === null || m === void 0 ? void 0 : m.length) !== 2) {
        return null;
    }
    if ($ok(m[1])) {
        const protocol = m[1].toLowerCase();
        if (!protocol.length) {
            return null;
        }
        if ($count(opts.acceptedProtocols)) {
            return $ok(opts.acceptedProtocols.find(v => v.toLowerCase() === protocol)) ? s : null;
        }
        return protocol === 'http' || protocol === 'https' ? s : null;
    }
    return m[1] !== null && opts.acceptsProtocolRelativeUrl ? s : null;
}
exports.$url = $url;
function $UUID(s) {
    if (!$isstring(s)) {
        return null;
    }
    return _regexvalidatedstring(types_1.uuidRegex, s);
}
exports.$UUID = $UUID;
function $isodate(s, format = tsdatecomp_1.TSDateForm.ISO8601) {
    let cps = null;
    if ($ok(s)) {
        if (s instanceof Date) {
            cps = (0, tsdatecomp_1.$components)(s);
        }
        else if (s instanceof tsdate_1.TSDate) {
            cps = s.toComponents();
        }
        else if ($isstring(s)) {
            cps = (0, tsdatecomp_1.$parsedatetime)($trim(s), format);
        } // we parse the string to verify it
    }
    return $ok(cps) ? (0, tsdatecomp_1.$components2string)(cps, format) : null;
}
exports.$isodate = $isodate;
function $address(a) {
    if (!$isobject(a)) {
        return null;
    }
    const city = $trim(a === null || a === void 0 ? void 0 : a.city);
    const country = (0, tsdefaults_1.$country)(a === null || a === void 0 ? void 0 : a.country);
    if (!$isstring(city) || !$length(city) || !$ok(country)) {
        return null;
    }
    let ret = Object.assign({}, a);
    ret.city = city;
    ret.country = country;
    return ret;
}
exports.$address = $address;
function $unsignedornull(n) {
    if (!$ok(n)) {
        return null;
    }
    if (typeof n === 'string') {
        n = parseInt(n, 10);
    }
    return $isunsigned(n) ? n : null;
}
exports.$unsignedornull = $unsignedornull;
function $unsigned(n, defaultValue = 0) {
    n = $unsignedornull(n);
    return $ok(n) ? n : defaultValue;
}
exports.$unsigned = $unsigned;
function $div(a, b) { return a / b | 0; }
exports.$div = $div;
function $string(v) {
    if (!$ok(v))
        return '';
    return typeof v === 'object' && 'toString' in v ? v.toString() : `${v}`;
}
exports.$string = $string;
function $strings(v) { return $ok(v) ? ($isarray(v) ? v : [v]) : []; }
exports.$strings = $strings;
function $totype(v) { return $ok(v) ? v : null; }
exports.$totype = $totype;
function $trim(s) { return $length(s) ? s.replace(/^\s*(\S*(\s+\S+)*)\s*$/, "$1") : ''; }
exports.$trim = $trim;
function $fpad2(v) { return $fpad(v, 2); }
exports.$fpad2 = $fpad2;
function $fpad3(v) { return $fpad(v, 3); }
exports.$fpad3 = $fpad3;
function $fpad4(v) { return $fpad(v, 4); }
exports.$fpad4 = $fpad4;
function $fpad(v, pad) { return v.toString().padStart(pad, '0'); }
exports.$fpad = $fpad;
// for now $ascii() does not mak any transliterations from
// non-latin languages like Greek
function $ascii(source) {
    const l = $length(source);
    if (!l)
        return '';
    let s = source.replace(/≠/g, "");
    s = s.normalize("NFD").replace(/[\u0300-\u036f]|\u00a8|\u00b4/g, "").normalize("NFKD"); // does most of the job
    // finally we will try to convert (or remove) the remaining non ascii characters
    return s.replace(/[^\x00-\x7F]/g, x => string_tables_1.FoundationASCIIConversion[x] || '');
}
exports.$ascii = $ascii;
function $count(a) { return $ok(a) && Array.isArray(a) ? a.length : 0; }
exports.$count = $count;
function $length(s) { return $ok(s) ? s.length : 0; }
exports.$length = $length;
function $lengthin(s, min = 0, max = types_1.INT_MAX) { const l = $length(s); return l >= min && l <= max; }
exports.$lengthin = $lengthin;
function $arraybuffer(buf) {
    const ret = new ArrayBuffer(buf.length);
    const view = new Uint8Array(ret);
    for (let i = 0; i < buf.length; ++i) {
        view[i] = buf[i];
    }
    return ret;
}
exports.$arraybuffer = $arraybuffer;
/*
    This is a map function where callback returns as null or undefined are
    flushed from the result
 */
function $map(a, callBack) {
    const ret = new Array();
    a === null || a === void 0 ? void 0 : a.forEach(e => {
        const v = callBack(e);
        if ($ok(v))
            ret.push(v);
    });
    return ret;
}
exports.$map = $map;
function $jsonobj(v) {
    if (v === null || v === undefined)
        return v;
    const t = typeof v;
    switch (t) {
        case 'object':
            return 'toJSON' in v ? v.toJSON() : v;
        case 'boolean':
        case 'number':
        case 'bigint':
        case 'string':
            return v;
        default:
            return undefined;
    }
}
exports.$jsonobj = $jsonobj;
function $keys(o) { return $ok(o) ? Object.getOwnPropertyNames(o) : []; }
exports.$keys = $keys;
// default filter supress undefined, null values and functions (methods for real class instances)
// if properties is not set, we use all the object keys to apply the filter
function $partial(a, opts = {}) {
    let ret = {};
    let n = 0;
    if ($ok(a)) {
        n = _fillObject('partial', ret, a, opts);
    }
    return [ret, n];
}
exports.$partial = $partial;
// TODO: review all AnyDictionary functions to make a specific file 
// and a more thorough specificaiton. Here, all null, undefined and function
// properties are removed
function $dict(source, keys) {
    const ret = {};
    if ($ok(source)) {
        _fillObject('dict', ret, source, { properties: keys });
    }
    return ret;
}
exports.$dict = $dict;
function $includesdict(source, dict, keys) {
    if ($ok(source)) {
        keys = $ok(keys) ? keys : $keys(dict);
        if (keys.length) {
            const v = source;
            for (let k of keys) {
                if (!(0, compare_1.$equal)(v[k], dict[k])) {
                    return false;
                }
            }
            return true;
        }
    }
    return false;
}
exports.$includesdict = $includesdict;
function $fusion(a, b, opts = {}) {
    if (!$ok(a)) {
        return $ok(b) ? $partial(b, opts.B) : [{}, 0];
    }
    else if (!$ok(b)) {
        return $partial(a, opts.A);
    }
    let [ret, n] = $partial(a, opts.A);
    let fopts = $ok(opts.B) ? Object.assign({}, opts.B) : {};
    if ($ok(opts.fusionArrays)) {
        fopts.fusionArrays = opts.fusionArrays;
    }
    if ($ok(opts.fusionObjects)) {
        fopts.fusionObjects = opts.fusionObjects;
    }
    n += _fillObject('fusion', ret, b, fopts);
    return [ret, n];
}
exports.$fusion = $fusion;
function $json(v, replacer = null, space = 2) { return JSON.stringify(v, replacer, space); }
exports.$json = $json;
// ===== private functions ===================================
function _regexvalidatedstring(regex, s) {
    const v = $trim(s);
    if (!v.length || !regex.test(v)) {
        return null;
    }
    return v;
}
function _fillObject(fn, destination, source, opts = {}) {
    let ret = 0;
    opts = Object.assign({}, opts);
    opts.properties = $ok(opts.properties) ? opts.properties : $keys(source);
    if (opts.properties.length) {
        const fusion_arrays = $isfunction(opts.fusionArrays);
        const fusion_objects = $isfunction(opts.fusionObjects);
        opts.filter = $isfunction(opts.filter) ? opts.filter : (v) => $ok(v) && typeof v !== 'function' ? v : undefined;
        for (let p of opts.properties) {
            if (!$isstring(p)) {
                throw `${fn}() needs to have valid string properties`;
            }
            const v = opts.filter(source[p]);
            if ($defined(v)) {
                if (fusion_arrays && $isarray(v) && $isarray(destination[p])) {
                    destination[p] = opts.fusionArrays(destination[p], v);
                }
                else if (fusion_objects && $isobject(v) && $isobject(destination[p])) {
                    destination[p] = opts.fusionObjects(destination[p], v);
                }
                else {
                    destination[p] = v;
                }
                ret++;
            }
        }
    }
    return ret;
}
//# sourceMappingURL=commons.js.map