/// <reference types="node" />
import { IncomingMessage, ServerResponse } from "http";
import { StringDictionary, uint16 } from "./types";
import { TSStaticWebSiteOptions } from "./tsservercomp";
/**
 * This is a minimal HTTP server class provided for testing
 *
 * You start a new API server on default port 3000 by a single line :
 *
 */
export declare type TSEndPoint = (url: URL, parameters: StringDictionary, req: IncomingMessage, resp: ServerResponse) => Promise<void>;
export declare type TSEndPoints = {
    [key in string]: TSEndPoint;
};
export declare type TSEndPointsDictionary = {
    [key: string]: TSEndPoints;
};
export interface TSServerOptions extends TSStaticWebSiteOptions {
    host?: string;
    port?: uint16;
    webSites?: StringDictionary;
    logInfos?: boolean;
}
export declare class TSServer {
    readonly host: string;
    readonly port: number;
    private static __server;
    private _httpServer;
    private _endPoints;
    private _sites;
    private _logger;
    private _logInfos;
    static start(endPoints: TSEndPointsDictionary, opts: TSServerOptions): Promise<void>;
    static clearCaches(): Promise<void>;
    static stop(): Promise<Error | undefined>;
    private constructor();
    private _start;
    private _clearCaches;
    private _stop;
}
export declare enum TSServerLogType {
    Log = "Info",
    Warning = "Warning",
    Error = "Error"
}
export declare type TSServerLogger = (server: TSServer, req: IncomingMessage | undefined, type: TSServerLogType, messages: string) => Promise<void>;
